@@ -475,7 +475,7 @@ CTCP_HANDLER(do_ping_reply)
 	/* Reply must be between time we started and right now */
 	if (orig < start_time || orig > t.tv_sec)
 	{
-		yell("hrm");
+		say("Invalid CTCP PING reply [%s] dropped.", cmd);
 		return NULL;
 	}
 
