@@ -689,6 +689,18 @@ static void 	handle_filedesc (Process *proc, int *fd, int hook_nonl, int hook_nl
 		}
 		case 0:		/* We didnt get a full line */
 		{
+			/* 
+			 * XXX XXX Major, world class hack here XXX XXX 
+			 * Since this problem has been addressed already
+			 * by newio2.c, it doesn't matter if we just fix the
+			 * symptom here since this code is all obsolete.
+			 */
+			if (exec_buffer[0] == '\n')
+			{
+				exec_buffer[0] = 0;
+				goto this_sucks;
+			}
+
 			     if (hook_nl == EXEC_LIST && proc->stdoutpc)
 				parse_line("EXEC", proc->stdoutpc, 
 						exec_buffer, 0, 0);
@@ -708,6 +720,7 @@ static void 	handle_filedesc (Process *proc, int *fd, int hook_nonl, int hook_nl
 		}
 		default:	/* We got a full line */
 		{
+this_sucks:
 			message_to(proc->refnum);
 			proc->counter++;
 
