@@ -1060,7 +1060,7 @@ BUILT_IN_COMMAND(oncmd)
 
 			/*
 			 * Otherwise, its an ambiguous type.  If they were 
-			 * trying to register the hook, then theyve already
+			 * trying to register the hook, then they've already
 			 * gotten the error message, just return;
 			 */
 			if (new_new_next_arg(args, &args, &type))
@@ -1069,10 +1069,14 @@ BUILT_IN_COMMAND(oncmd)
 			/*
 			 * Ok.  So they probably want a listing.
 			 */
-			say("ON listings:");
 			len = strlen(func);
 			while (!my_strnicmp(func, hook_functions[first].name, len))
-				show_list(first), first++;
+			{
+			    if (!show_list(first))
+				say("The %s list is empty.", 
+					hook_functions[first].name);
+			    first++;
+			}
 
 			return;
 		}
