@@ -68,6 +68,8 @@ static	char	*input_prompt;
 #define INPUT_PROMPT 		current_screen->input_prompt
 #define INPUT_PROMPT_LEN 	current_screen->input_prompt_len
 #define INPUT_LINE 		current_screen->input_line
+#define CUT_BUFFER		cut_buffer
+#define SET_CUT_BUFFER(x)	malloc_strcpy(&CUT_BUFFER, x);
 
 #define BUILT_IN_KEYBINDING(x) void x (char key, char *string)
 
@@ -435,24 +437,24 @@ void 	change_input_prompt (int direction)
 		strcpy(INPUT_BUFFER, last_input_screen->saved_input_buffer);
 		THIS_POS = last_input_screen->saved_buffer_pos;
 		MIN_POS = last_input_screen->saved_min_buffer_pos;
-		*last_input_screen->saved_input_buffer = '\0';
+		*last_input_screen->saved_input_buffer = 0;
 		last_input_screen->saved_buffer_pos = 0;
 		last_input_screen->saved_min_buffer_pos = 0;
-		update_input(UPDATE_ALL);
 	}
 
 	else if (direction == -1)
-		update_input(UPDATE_ALL);
+		;
 
 	else if (!last_input_screen->promptlist->next)
 	{
 		strcpy(last_input_screen->saved_input_buffer, INPUT_BUFFER);
 		last_input_screen->saved_buffer_pos = THIS_POS;
 		last_input_screen->saved_min_buffer_pos = MIN_POS;
-		*INPUT_BUFFER = '\0';
+		*INPUT_BUFFER = 0;
 		THIS_POS = MIN_POS = 0;
-		update_input(UPDATE_ALL);
 	}
+
+	update_input(UPDATE_ALL);
 }
 
 /* input_move_cursor: moves the cursor left or right... got it? */
@@ -495,13 +497,13 @@ void	set_input (char *str)
  */
 char	*get_input (void)
 {
-	return (&(MIN_CHAR));
+	return &MIN_CHAR;
 }
 
 /* init_input: initialized the input buffer by clearing it out */
 void	init_input (void)
 {
-	*INPUT_BUFFER = (char) 0;
+	*INPUT_BUFFER = 0;
 	THIS_POS = MIN_POS;
 }
 
@@ -529,6 +531,8 @@ void	set_input_prompt (char *prompt)
 }
 
 
+#define WHITESPACE(x) (my_isspace(x) || ispunct(x))
+
 /* 
  * Why did i put these in this file?  I dunno.  But i do know that the ones 
  * in edit.c didnt have to be here, and i knew the ones that were here DID 
@@ -544,10 +548,14 @@ BUILT_IN_KEYBINDING(input_forward_word)
 {
 	cursor_to_input();
 
-	while ((my_isspace(THIS_CHAR) || ispunct(THIS_CHAR)) && (THIS_CHAR))
-			THIS_POS++;
-	while (!(ispunct(THIS_CHAR) || my_isspace(THIS_CHAR)) && (THIS_CHAR))
-			THIS_POS++;
+	/* Move to the end of the current word to the whitespace */
+	while (THIS_CHAR && !WHITESPACE(THIS_CHAR))
+		THIS_POS++;
+
+	/* Move past the whitespace to the start of the next word */
+	while (THIS_CHAR && WHITESPACE(THIS_CHAR))
+		THIS_POS++;
+
 	update_input(UPDATE_JUST_CURSOR);
 }
 
@@ -555,123 +563,106 @@ BUILT_IN_KEYBINDING(input_forward_word)
 BUILT_IN_KEYBINDING(input_backward_word)
 {
 	cursor_to_input();
-	while ((THIS_POS > MIN_POS) && 
-			(my_isspace(PREV_CHAR) || ispunct(PREV_CHAR)))
+
+	/* If already at the start of a word, move back a position */
+	if (!WHITESPACE(THIS_CHAR) && WHITESPACE(PREV_CHAR))
 		THIS_POS--;
-	while ((THIS_POS > MIN_POS) && 
-			!(ispunct(PREV_CHAR) || my_isspace(PREV_CHAR)))
+
+	/* Move to the start of the current whitespace */
+	while ((THIS_POS > MIN_POS) && WHITESPACE(THIS_CHAR))
 		THIS_POS--;
 
+	/* Move to the start of the current word */
+	while ((THIS_POS > MIN_POS) && !WHITESPACE(THIS_CHAR))
+		THIS_POS--;
+
+	/* If we overshot our goal, then move forward */
+	/* But NOT if at start of input line! (July 6th, 1999) */
+	if ((THIS_POS > MIN_POS) && WHITESPACE(THIS_CHAR))
+		THIS_POS++;
+
 	update_input(UPDATE_JUST_CURSOR);
 }
 
-/* input_delete_character: deletes a character from the input line */
-BUILT_IN_KEYBINDING(input_delete_character)
+static void	input_delete_char_from_screen (void)
 {
-	cursor_to_input();
-	if (THIS_CHAR)
+	/*
+	 * Remove the current character from the screen's display.
+	 *
+	 * If we cannot do a character delete then we do a wholesale
+	 * redraw of the input line (ugh). 
+	 */
+	if (!(termfeatures & TERM_CAN_DELETE))
+		update_input(UPDATE_FROM_CURSOR);
+	else
 	{
-		char	*ptr = (char *) 0;
 		int	pos;
 
-		malloc_strcpy(&ptr, &(NEXT_CHAR));
-		strcpy(&(THIS_CHAR), ptr);
-		new_free(&ptr);
+		/*
+		 * Delete the character.  This is the simple part.
+		 */
+		term_delete(1);
 
 		/*
-		 * If term_delete fails or isnt available, then we
-		 * do a whole-sale redraw of the input line (ugh)
-		 * Otherwise, we find the character that is now on
-		 * the screen and write it.
+		 * So right now we have a blank space at the right of the
+		 * screen.  If there is a character in the input buffer that
+		 * is out in that position, we need to find it and display it.
 		 */
-		if (!(termfeatures & TERM_CAN_DELETE))
-			update_input(UPDATE_FROM_CURSOR);
-		else
+		pos = INPUT_ONSCREEN + last_input_screen->co - 1;
+		if (pos < strlen(INPUT_BUFFER))
 		{
-			term_delete(1);
-			pos = INPUT_ONSCREEN + last_input_screen->co - 1;
-			if (pos < strlen(INPUT_BUFFER))
-			{
-				term_move_cursor(last_input_screen->co - 1, INPUT_LINE);
-				term_putchar(INPUT_BUFFER[pos]);
-				term_move_cursor(INPUT_CURSOR, INPUT_LINE);
-			}
-			update_input(NO_UPDATE);
+			term_move_cursor(last_input_screen->co - 1, INPUT_LINE);
+			term_putchar(INPUT_BUFFER[pos]);
+			term_move_cursor(INPUT_CURSOR, INPUT_LINE);
 		}
+
+		/* XXX - Very possibly, this is pointless */
+		update_input(NO_UPDATE);
 	}
 }
 
+/*
+ * input_delete_character -- Deletes the character currently under the
+ * 			     input cursor.
+ */
+BUILT_IN_KEYBINDING(input_delete_character)
+{
+	cursor_to_input();
 
-/* input_backspace: does a backspace in the input buffer */
+	/*
+	 * If we are at the end of the input buffer, the delete key has
+	 * no effect.
+	 */
+	if (!THIS_CHAR)
+		return;
+
+	/* 
+	 * Remove the current character from the logical buffer
+	 * and also from the screen.
+	 */
+	strcpy(&THIS_CHAR, LOCAL_COPY(&NEXT_CHAR));
+	input_delete_char_from_screen();
+}
+
+
+/*
+ * input_backspace -- Basically a combination of backward_character and
+ *		      delete_character.  No, this is not significantly
+ *		      more expensive than the old way.
+ */
 BUILT_IN_KEYBINDING(input_backspace)
 {
 	cursor_to_input();
 
-	/* If we are not at the beginning of the input buffer */
+	/* Only works when not at start of the input buffer */
 	if (THIS_POS > MIN_POS)
 	{
-		char	*ptr = (char *) 0;
-		int	pos;
-
 		/*
-		 * XXX - we do an alloca just to remove the previous
-		 * character in the buffer.  We should be calling a
-		 * overlap-safe strcpy() here instead.
+		 * Back up the logical buffer cursor, the logical screen
+		 * cursor, the real screen cursor, and then do a delete.
 		 */
-		/*
-		 * First we remove the character from the logical buffer
-		 */
-		ptr = LOCAL_COPY(&THIS_CHAR);
-		strcpy(&(PREV_CHAR), ptr);
-		THIS_POS--;
-
-		/*
-		 * Now we remove the character from the physical screen.
-		 */
-		term_cursor_left();
-
-		/*
-		 * If we're in the middle of the string (THIS_CHAR isnt 0)
-		 * then we delete the character under the cursor  (we just 
-		 * backed on to it.  Then we look to see if there is more 
-		 * of the input buffer than we see on the screen.  If there
-		 * is, then we grab the "next char" that is off the screen
-		 * and put it at the end.
-		 */
-		if (THIS_CHAR)
-		{
-			if (!(termfeatures & TERM_CAN_DELETE))
-				update_input(UPDATE_FROM_CURSOR);
-			else
-			{
-				term_delete(1);
-				pos = INPUT_ONSCREEN + last_input_screen->co - 1;
-				/*
-				 * If the prompt is visible right now,
-				 * then we have to cope with that.
-				 */
-				if (START_ZONE == WIDTH)
-					pos -= INPUT_PROMPT_LEN;
-				if (pos < strlen(INPUT_BUFFER))
-				{
-					term_move_cursor(last_input_screen->co - 1, INPUT_LINE);
-					term_putchar(INPUT_BUFFER[pos]);
-				}
-				update_input(UPDATE_JUST_CURSOR);
-			}
-		}
-
-		/*
-		 * We're at the end, so we just blank out the current
-		 * character (the one we're deleting), and move the cursor
-		 * back one character (the putchar advances it).
-		 */
-		else
-		{
-			term_putchar(' ');
-			term_cursor_left();
-			update_input(NO_UPDATE);
-		}
+		backward_character(0, NULL);
+		input_delete_character(0, NULL);
 	}
 }
 
@@ -697,51 +688,86 @@ BUILT_IN_KEYBINDING(input_end_of_line)
 	update_input(UPDATE_JUST_CURSOR);
 }
 
-BUILT_IN_KEYBINDING(input_delete_to_previous_space)
+/*
+ * This removes every character from the 'anchor' position to the current
+ * position from the input line, and puts it into the cut buffer.  It does
+ * the requisite redraw as well.
+ */
+void	cut_input (int anchor)
 {
-	int	old_pos;
-	char	c;
+	char *	buffer;
+	int	size;
 
-	cursor_to_input();
-	old_pos = THIS_POS;
-	c = THIS_CHAR;
-
-	while (!my_isspace(THIS_CHAR) && THIS_POS >= MIN_POS)
-		THIS_POS--;
-
-	if (THIS_POS < old_pos)
+	if (anchor < THIS_POS)
 	{
-		strcpy(&(NEXT_CHAR), &(INPUT_BUFFER[old_pos]));
-		THIS_POS++;
+		buffer = alloca((size = THIS_POS - anchor));
+		strmcpy(buffer, &INPUT_BUFFER[anchor], size);
+		malloc_strcpy(&cut_buffer, buffer);
+
+		buffer = LOCAL_COPY(&THIS_CHAR);
+		INPUT_BUFFER[anchor] = 0;
+		ADD_TO_INPUT(buffer);
+		THIS_POS = anchor;
+	}
+	else
+	{
+		buffer = alloca((size = anchor - THIS_POS));
+		strmcpy(buffer, &THIS_CHAR, size);
+		malloc_strcpy(&cut_buffer, buffer);
+
+		buffer = LOCAL_COPY(&INPUT_BUFFER[anchor]);
+		THIS_CHAR = 0;
+		ADD_TO_INPUT(buffer);
 	}
 
-	update_input(UPDATE_FROM_CURSOR);
+	update_input(UPDATE_ALL);
+}
+
+/*
+ * To visualize:
+ *
+ *		This is the input buffer
+ *			  ^			(the input cursor)
+ * orig_pos points to the 'e'.
+ * c is an 'e'.
+ * THIS_POS is moved to the space between the 's' and the 't'.
+ * The input buffer is changed to:
+ *
+ *		This is e input buffer
+ *			^			(the input cursor)
+ */
+BUILT_IN_KEYBINDING(input_delete_to_previous_space)
+{
+	int	anchor;
+
+	cursor_to_input();
+
+	if (THIS_POS <= MIN_POS)
+		return;
+
+	anchor = THIS_POS;
+	while (THIS_POS > MIN_POS && !my_isspace(PREV_CHAR))
+		backward_character(0, NULL);
+	cut_input(anchor);
 }
 
 /*
  * input_delete_previous_word: deletes from the cursor backwards to the next
- * space character. 
+ * space character.  This is probably going to be the same effect as 
+ * delete_to_previous_space, but hey -- you know.
  */
 BUILT_IN_KEYBINDING(input_delete_previous_word)
 {
-	int	old_pos;
-	char	c;
+	int	anchor;
 
 	cursor_to_input();
-	old_pos = THIS_POS;
-	while ((THIS_POS > MIN_POS) && 
-			(my_isspace(PREV_CHAR) || ispunct(PREV_CHAR)))
-		THIS_POS--;
-	while ((THIS_POS > MIN_POS) && 
-			!(ispunct(PREV_CHAR) || my_isspace(PREV_CHAR)))
-		THIS_POS--;
 
-	c = INPUT_BUFFER[old_pos];
-	INPUT_BUFFER[old_pos] = (char) 0;
-	malloc_strcpy(&cut_buffer, &THIS_CHAR);
-	INPUT_BUFFER[old_pos] = c;
-	strcpy(&(THIS_CHAR), &(INPUT_BUFFER[old_pos]));
-	update_input(UPDATE_FROM_CURSOR);
+	if (THIS_POS <= MIN_POS)
+		return;
+
+	anchor = THIS_POS;
+	input_backward_word(0, NULL);
+	cut_input(anchor);
 }
 
 /*
@@ -750,27 +776,16 @@ BUILT_IN_KEYBINDING(input_delete_previous_word)
  */
 BUILT_IN_KEYBINDING(input_delete_next_word)
 {
-	int	pos;
-	char	*ptr = (char *) 0,
-		c;
+	int	anchor;
 
 	cursor_to_input();
-	pos = THIS_POS;
-	while ((my_isspace(INPUT_BUFFER[pos]) || 
-			ispunct(INPUT_BUFFER[pos])) && INPUT_BUFFER[pos])
-		pos++;
-	while (!(ispunct(INPUT_BUFFER[pos]) || 
-			my_isspace(INPUT_BUFFER[pos])) && INPUT_BUFFER[pos])
-		pos++;
 
-	c = INPUT_BUFFER[pos];
-	INPUT_BUFFER[pos] = (char) 0;
-	malloc_strcpy(&cut_buffer, &(THIS_CHAR));
-	INPUT_BUFFER[pos] = c;
-	malloc_strcpy(&ptr, &(INPUT_BUFFER[pos]));
-	strcpy(&(THIS_CHAR), ptr);
-	new_free(&ptr);
-	update_input(UPDATE_FROM_CURSOR);
+	if (!THIS_CHAR)
+		return;
+
+	anchor = THIS_POS;
+	input_forward_word(0, NULL);
+	cut_input(anchor);
 }
 
 /*
@@ -781,16 +796,19 @@ BUILT_IN_KEYBINDING(input_add_character)
 {
 	int	display_flag = NO_UPDATE;
 
+	cursor_to_input();
+
 	if (last_input_screen->promptlist)
 		term_echo(last_input_screen->promptlist->echo);
 
-	cursor_to_input();
+	/* Don't permit the input buffer to get too big. */
 	if (THIS_POS >= INPUT_BUFFER_SIZE)
 	{
 		term_echo(1);
 		return;
 	}
 
+	/* XXX Is a get_int_var really neccesary here? */
 	if (get_int_var(INSERT_MODE_VAR))
 	{
 		/*
@@ -800,11 +818,17 @@ BUILT_IN_KEYBINDING(input_add_character)
 		{
 			char	*ptr;
 
-			ptr = alloca(strlen(&(THIS_CHAR)) + 1);
-			strcpy(ptr, &(THIS_CHAR));
+			/*
+			 * Add to logical buffer
+			 */
+			ptr = LOCAL_COPY(&(THIS_CHAR));
 			THIS_CHAR = key;
 			NEXT_CHAR = 0;
 			ADD_TO_INPUT(ptr);
+
+			/*
+			 * Add to display screen
+			 */
 			if (termfeatures & TERM_CAN_INSERT)
 				term_insert(key);
 			else
@@ -818,11 +842,18 @@ BUILT_IN_KEYBINDING(input_add_character)
 		}
 		else
 		{
+			/*
+			 * Add to logical buffer
+			 */
 			THIS_CHAR = key;
 			NEXT_CHAR = 0;
+
+			/* Add to display screen */
 			term_putchar(key);
 		}
 	}
+
+	/* Overstrike mode.  Much simpler. */
 	else
 	{
 		if (THIS_CHAR == 0)
@@ -839,8 +870,9 @@ BUILT_IN_KEYBINDING(input_add_character)
 /* input_clear_to_eol: erases from the cursor to the end of the input buffer */
 BUILT_IN_KEYBINDING(input_clear_to_eol)
 {
+	/* This doesnt really speak to the implementation, but it works.  */
 	cursor_to_input();
-	malloc_strcpy(&cut_buffer, &(THIS_CHAR));
+	malloc_strcpy(&cut_buffer, &THIS_CHAR);
 	THIS_CHAR = 0;
 	term_clear_to_eol();
 	update_input(NO_UPDATE);
@@ -852,15 +884,15 @@ BUILT_IN_KEYBINDING(input_clear_to_eol)
  */
 BUILT_IN_KEYBINDING(input_clear_to_bol)
 {
-	char	*ptr = (char *) 0;
+	char	c = THIS_CHAR;
 
 	cursor_to_input();
-	malloc_strcpy(&cut_buffer, &(MIN_CHAR));
-	cut_buffer[THIS_POS - MIN_POS] = (char) 0;
-	malloc_strcpy(&ptr, &(THIS_CHAR));
-	MIN_CHAR = (char) 0;
-	ADD_TO_INPUT(ptr);
-	new_free(&ptr);
+
+	THIS_CHAR = 0;
+	malloc_strcpy(&cut_buffer, &MIN_CHAR);
+	THIS_CHAR = c;
+
+	set_input(LOCAL_COPY(&THIS_CHAR));
 	THIS_POS = MIN_POS;
 	term_move_cursor(INPUT_PROMPT_LEN, INPUT_LINE);
 	term_clear_to_eol();
@@ -873,8 +905,9 @@ BUILT_IN_KEYBINDING(input_clear_to_bol)
 BUILT_IN_KEYBINDING(input_clear_line)
 {
 	cursor_to_input();
+
 	malloc_strcpy(&cut_buffer, INPUT_BUFFER + MIN_POS);
-	MIN_CHAR = (char) 0;
+	MIN_CHAR = 0;
 	THIS_POS = MIN_POS;
 	term_move_cursor(INPUT_PROMPT_LEN, INPUT_LINE);
 	term_clear_to_eol();
@@ -905,6 +938,7 @@ BUILT_IN_KEYBINDING(input_transpose_characters)
 		 * Else if the input buffer has at least two
 		 * characters in it (and implicity we're at the end)
 		 * then swap the two characters before the cursor
+		 * XXX This strlen() is probably unnecesary.
 		 */
 		else if (strlen(INPUT_BUFFER) > MIN_POS + 2)
 		{
@@ -961,22 +995,21 @@ BUILT_IN_KEYBINDING(refresh_inputline)
  */
 BUILT_IN_KEYBINDING(input_yank_cut_buffer)
 {
-	char	*ptr = (char *) 0;
+	char	*ptr = NULL;
 
-	if (cut_buffer)
-	{
-		malloc_strcpy(&ptr, &(THIS_CHAR));
-		/* Ooops... */
-		THIS_CHAR = 0;
-		ADD_TO_INPUT(cut_buffer);
-		ADD_TO_INPUT(ptr);
-		new_free(&ptr);
-		update_input(UPDATE_FROM_CURSOR);
-		THIS_POS += strlen(cut_buffer);
-		if (THIS_POS > INPUT_BUFFER_SIZE)
-			THIS_POS = INPUT_BUFFER_SIZE;
-		update_input(UPDATE_JUST_CURSOR);
-	}
+	if (!cut_buffer)
+		return;
+
+	ptr = LOCAL_COPY(&THIS_CHAR);
+	THIS_CHAR = 0;
+	ADD_TO_INPUT(cut_buffer);
+	ADD_TO_INPUT(ptr);
+	update_input(UPDATE_FROM_CURSOR);
+
+	THIS_POS += strlen(cut_buffer);
+	if (THIS_POS > INPUT_BUFFER_SIZE)
+		THIS_POS = INPUT_BUFFER_SIZE;
+	update_input(UPDATE_JUST_CURSOR);
 }
 
 
@@ -1012,19 +1045,20 @@ BUILT_IN_KEYBINDING(toggle_insert_mode)
 
 BUILT_IN_KEYBINDING(send_line)
 {
-	int	server;
-	WaitPrompt	*OldPrompt;
+	int	server = from_server;
+	char *	line = LOCAL_COPY(get_input());
 
-	server = from_server;
 	from_server = get_window_server(0);
 	unhold_a_window(last_input_screen->current_window);
+	MIN_CHAR = 0;
+	THIS_POS = MIN_POS;
 
 	if (last_input_screen->promptlist && 
 		last_input_screen->promptlist->type == WAIT_PROMPT_LINE)
 	{
-		OldPrompt = last_input_screen->promptlist;
-		(*OldPrompt->func)(OldPrompt->data, get_input());
-		set_input(empty_string);
+		WaitPrompt *	OldPrompt = last_input_screen->promptlist;
+
+		(*OldPrompt->func)(OldPrompt->data, line);
 		last_input_screen->promptlist = OldPrompt->next;
 		new_free(&OldPrompt->data);
 		new_free(&OldPrompt->prompt);
@@ -1033,22 +1067,18 @@ BUILT_IN_KEYBINDING(send_line)
 	}
 	else
 	{
-		char	*line = get_input(),
-			*tmp;
-
-		tmp = (char *)alloca(strlen(line) + 1);
-		strcpy(tmp, line);
-		set_input(empty_string); /* Why do i know this is wrong? */
+		/* Clear the input line before dispatching the command */
 		update_input(UPDATE_ALL);
 
-		if (do_hook(INPUT_LIST, "%s", tmp))
+		if (do_hook(INPUT_LIST, "%s", line))
 		{
 			if (get_int_var(INPUT_ALIASES_VAR))
-				parse_line(NULL, tmp, empty_string, 1, 0);
+				parse_line(NULL, line, empty_string, 1, 0);
 			else
-				parse_line(NULL, tmp, NULL, 1, 0);
+				parse_line(NULL, line, NULL, 1, 0);
 		}
 	}
+
 	from_server = server;
 }
 
@@ -1067,27 +1097,27 @@ BUILT_IN_KEYBINDING(quote_char)
  */
 BUILT_IN_KEYBINDING(insert_bold)
 {
-	input_add_character (BOLD_TOG, string);
+	input_add_character(BOLD_TOG, string);
 }
 
 BUILT_IN_KEYBINDING(insert_reverse)
 {
-	input_add_character (REV_TOG, string);
+	input_add_character(REV_TOG, string);
 }
 
 BUILT_IN_KEYBINDING(insert_underline)
 {
-	input_add_character (UND_TOG, string);
+	input_add_character(UND_TOG, string);
 }
 
 BUILT_IN_KEYBINDING(highlight_off)
 {
-	input_add_character (ALL_OFF, string);
+	input_add_character(ALL_OFF, string);
 }
 
 BUILT_IN_KEYBINDING(insert_blink)
 {
-	input_add_character (BLINK_TOG, string);
+	input_add_character(BLINK_TOG, string);
 }
 
 /* type_text: the BIND function TYPE_TEXT */
@@ -1133,9 +1163,9 @@ BUILT_IN_KEYBINDING(cpu_saver_on)
 void	edit_char (u_char key)
 {
 	void		(*func) 	(char, char *) = NULL;
-	char		*ptr = NULL;
+	char *		ptr = NULL;
 	u_char		extended_key;
-	WaitPrompt 	*oldprompt;
+	WaitPrompt *	oldprompt;
 	u_char		dummy[2];
 	int		xxx_return = 0;		/* XXXX Need i say more? */
 
@@ -1166,6 +1196,10 @@ void	edit_char (u_char key)
 		xxx_return = 1;
 	}
 
+	/* 
+	 * This is only used by /pause to see when a keypress event occurs,
+	 * but not to impact how that keypress is handled at all.
+	 */
 	if (last_input_screen->promptlist && 
 		last_input_screen->promptlist->type == WAIT_PROMPT_DUMMY)
 	{
@@ -1180,6 +1214,7 @@ void	edit_char (u_char key)
 	if (xxx_return)
 		return;
 
+	/* If the high bit is set, mangle it as neccesary. */
 	if (key & 0x80)
 	{
 		if (meta_mode)
@@ -1193,12 +1228,14 @@ void	edit_char (u_char key)
 
 	extended_key = key;
 
-	/* did we just hit the quote character? */
+	/* If we just hit the quote character, add this character literally */
 	if (last_input_screen->quote_hit)
 	{
 		last_input_screen->quote_hit = 0;
 		input_add_character(extended_key, empty_string);
 	}
+
+	/* Otherwise find out what its binding is and dispatch it. */
 	else
 	{
 		int	m = last_input_screen->meta_hit;
@@ -1229,21 +1266,16 @@ BUILT_IN_COMMAND(type)
 	int	c;
 	char	key;
 
-	while (*args)
+	for (; *args; args++)
 	{
 		if (*args == '^')
 		{
-			switch (*(++args))
-			{
-			    case '?':
-			    {
+			args++;
+			if (*args == '?')
 				key = '\177';
-				args++;
-				break;
-			    }
-			    default:
-			    {
-				c = *(args++);
+			else
+			{
+				c = *args;
 				if (islower(c))
 					c = toupper(c);
 				if (c < 64)
@@ -1252,17 +1284,14 @@ BUILT_IN_COMMAND(type)
 					return;
 				}
 				key = c - 64;
-				break;
-			    }
 			}
 		}
-		else if (*args == '\\')
+		else 
 		{
-			key = *++args;
-			args++;
+			if (*args == '\\')
+				args++;
+			key = *args;
 		}
-		else
-			key = *(args++);
 
 		edit_char(key);
 	}
