@@ -13,13 +13,14 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC4pre2.003";
+const char irc_version[] = "EPIC4pre2.004-19990718";
+const char useful_info[] = "epic4 pre2 004";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "19990224";
+const char internal_version[] = "19990718";
 
 
 #define __need_putchar_x__
@@ -182,7 +183,7 @@ fd_set		readables, writables;
 
 
 static		char	switch_help[] =
-"Usage: irc [switches] [nickname] [server list]                       \n\
+"Usage: epic [switches] [nickname] [server list]                      \n\
   The [nickname] can be up to 30 characters long                      \n\
   The [server list] are one or more server descriptions               \n\
   The [switches] are zero or more of the following:                   \n\
@@ -694,7 +695,7 @@ static	void	parse_args (int argc, char **argv)
 	 */
 	if (!LocalHostName)
 	{
-		if (gethostname(hostname, NAME_LEN))
+		if (gethostname(hostname, NAME_LEN) || strlen(hostname) == 0)
 		{
 			printf("I don't know what your hostname is and I can't do much without it.\n");
 			exit(1);
@@ -782,6 +783,8 @@ static	struct	timeval cursor_timeout,
 		 * in actually calculating when the next minute will tick, 
 		 * but that will be offset by the fact that we will only
 		 * call select() once a minute instead of 4 times.
+		 * Plus you get the benefit of the clock actually changing
+		 * on time rather than up to 15 seconds later.
 		 */
 		clock_timeout.tv_usec = 0L;
 
@@ -1132,7 +1135,7 @@ void	reset_clock (char *unused)
 }
 
 
-int main (int argc, char *argv[])
+int 	main (int argc, char *argv[])
 {
 #ifdef SOCKS
 	SOCKSinit(argv[0]);
