@@ -11,15 +11,6 @@
 static	char	rcsid[] = "@(#)$Id: ircaux.c,v 1.24 1994/07/02 02:32:13 mrg Exp $";
 #endif
 
-/*
- * This is on for NR12 because NR12 is still a testers-only release.
- * Once we find the last few problems, we will #undef this for the final
- * release.  And it will always be able to be turned on, anyhow.  Basically 
- * when this is #define'd, we track filenames and line numbers for 
- * double-free's.
- */
-#define MALLOC_DEBUG
-
 #include "irc.h"
 #include "screen.h"
 #include <pwd.h>
@@ -30,7 +21,6 @@ static	char	rcsid[] = "@(#)$Id: ircaux.c,v 1.24 1994/07/02 02:32:13 mrg Exp $";
 #include "vars.h"
 #include "alias.h"
 
-
 /*
  * This is the basic overhead for every malloc allocation (8 bytes).
  * The size of the allocation and a "magic" sequence are retained here.
@@ -42,19 +32,11 @@ typedef struct _mo_money
 {
 	unsigned magic;
 	int size;
-#ifdef MALLOC_DEBUG
-	char file[8];
-	int line;
-#endif
 } MO;
 
 #define mo_ptr(ptr) ((MO *)( (char *)(ptr) - sizeof(MO) ))
 #define alloc_size(ptr) ((mo_ptr(ptr))->size)
 #define magic(ptr) ((mo_ptr(ptr))->magic)
-#ifdef MALLOC_DEBUG
-#define line(ptr) ((mo_ptr(ptr))->line)
-#define filexxx(ptr) ((mo_ptr(ptr))->file)
-#endif
 
 #define FREED_VAL -3
 #define ALLOC_MAGIC (unsigned long)0x7fbdce70
@@ -75,10 +57,6 @@ void *	really_new_malloc (size_t size, char *fn, int line)
 	ptr += sizeof(MO);
 	magic(ptr) = ALLOC_MAGIC;
 	alloc_size(ptr) = size;
-#ifdef MALLOC_DEBUG
-	strmcpy(filexxx(ptr), fn, 7);
-	line(ptr) = line;
-#endif
 	return ptr;
 }
 
@@ -97,10 +75,6 @@ int	debug_new_free(void **ptr, char *fn, int line)
 			return ALREADY_FREED;
 
 		alloc_size(*ptr) = FREED_VAL;
-#ifdef MALLOC_DEBUG
-		strmcpy(filexxx(*ptr), fn, 7);
-		line(*ptr) = line;
-#endif
 		free((void *)(mo_ptr(*ptr)));
 		*ptr = (void *) 0;
 	}
@@ -133,12 +107,8 @@ void *	really_new_free(void **ptr, char *fn, int line)
 
 		case ALREADY_FREED:
 		{
-#ifdef MALLOC_DEBUG
-		    panic("Memory previously freed at [%s:%d] freed again at [%s:%d]", filexxx(*ptr), line(*ptr), fn, line);
-#else
 		    panic("free()d the same address twice from [%s/%d].", 
 			fn, line);
-#endif
 		}
 	    }
 	}
@@ -2870,8 +2840,8 @@ static	unsigned long	randm (unsigned long l)
 	/* patch from Sarayan to make $rand() better */
 static	const	long	RAND_A = 16807L;
 static	const	long	RAND_M = 2147483647L;
-static	const		RAND_Q = 127773L;
-static	const	int	RAND_R = 2836L;
+static	const	long	RAND_Q = 127773L;
+static	const	long	RAND_R = 2836L;
 static		u_long	z = 0;
 		long	t;
 
@@ -2889,7 +2859,7 @@ static		u_long	z = 0;
 	}
 	else
 	{
-		if (l < 0)
+		if ((long) l < 0)
 			z = (u_long) getuid();
 		else
 			z = l;
@@ -2924,7 +2894,7 @@ static	unsigned long randt (unsigned long l)
 	t = (t1 & 65535) * 65536 + (t2 & 65535);
 	return t;
 #else
-	return randm();
+	return randm(0);
 #endif
 }
 
@@ -2975,3 +2945,83 @@ unsigned long	random_number (unsigned long l)
 	}
 }
 
+/*
+ * urlencode: converts non-alphanumeric characters to hexidecimal codes
+ * Contributed by SrFrog
+ */
+char *	urlencode (const char *s)
+{
+	static const char unsafe[] = "`'!@#$%^&*(){}<>~|\\\";? ,/";
+	static const char hexnum[] = "0123456789ABCDEF";
+	const char *p1;
+	char *	p2;
+	char *	retval;
+
+	if (!s || !*s)
+		return NULL;
+
+	retval = new_malloc(strlen(s) * 3 + 1);
+
+	for (p1 = s, p2 = retval; *p1; p1++)
+	{
+		if (strchr(unsafe, *p1))
+		{
+			unsigned c = (unsigned) *p1;
+
+			*p2++ = '%';
+			*p2++ = hexnum[c >> 4];
+			*p2++ = hexnum[c & 0x0f];
+		}
+		else
+			*p2++ = *p1;
+	}
+	*p2 = 0;
+
+	return retval;
+}
+
+#define XTOI(x) 					\
+( 							\
+	((x) >= '0' && (x) <= '9') 			\
+		? ((x) - '0') 				\
+		: ( ((x) >= 'A' && (x) <= 'F')		\
+		    ? ((x) - 'A') 			\
+		    : ( ((x) >= 'a' && (x) <= 'f')	\
+			?  ((x) - 'a') 			\
+			: -1				\
+		      )					\
+		  )					\
+)
+
+char *	urldecode (char *s)
+{
+	const char *p1;
+	char *	p2;
+	size_t	len;
+	char *	retval;
+	int	val1;
+	int	val2;
+
+	if (!s || !*s)
+		return NULL;
+
+	len = strlen(s);
+	retval = alloca(len + 1);
+
+	for (p1 = s, p2 = retval; *p1; p1++, p2++)
+	{
+		if (*p1 == '%' && 
+		    ((p1[1] && ((val1 = XTOI(p1[1])) != -1)) && 
+		     (p1[2] && ((val2 = XTOI(p1[2])) != -1))))
+		{
+			p1++, p1++;
+			*p2 = (val1 * 16) + val2;
+		}
+		else
+			*p2 = *p1;
+	}
+
+	*p2 = 0;
+	return strcpy(s, retval);
+}
+
