@@ -233,6 +233,7 @@ void	resize_metamap (int new_size)
 	{
 		say("You can't set META_STATES to less than 5.");
 		set_int_var(META_STATES_VAR, 5);
+		new_size = 5;
 	}
 
 	if (old_size == new_size)
@@ -654,7 +655,8 @@ void	copy_redux (const uc *orig, uc *result)
 		switch (toupper(*ptr))
 		{
 			case 0:			/* ^<nul> is ^ */
-				*result = '^';
+				*result++ = '^';
+				*result = 0;
 				return;
 			case '?':		/* ^? is DEL */
 				*result = 0177;
