@@ -18,6 +18,7 @@
 #include "screen.h"
 #include "window.h"
 #include "server.h"
+#include "list.h"
 
 typedef struct nick_stru
 {
@@ -88,16 +89,18 @@ static	void	check_mode_list_join (const char *);
 
 struct modelist
 {
-	char	*chan;
-	char	*mode;
 	struct modelist *next;
-}	*mode_list = (struct modelist *) 0;
+	char *		chan;
+	char *		mode;
+};
+typedef struct modelist Modelist;
+static	Modelist *	mode_list = NULL;
 
 /* channel_list: list of all the channels you are currently on */
-static	Channel *channel_list = (Channel *) 0;
+static	Channel *	channel_list = NULL;
 
 /* For new eu2.9 !channels */
-static	char	new_channel_format[BIG_BUFFER_SIZE];
+static	char		new_channel_format[BIG_BUFFER_SIZE];
 
 
 /*
@@ -509,8 +512,21 @@ void 	add_userhost_to_channel (const char *channel, const char *nick, int server
 {
 	Nick *new_n;
 
-	if ((new_n = find_nick(server, channel, nick)))
-		malloc_strcpy(&new_n->userhost, uh);
+	while (!(new_n = find_nick(server, channel, nick)))
+	{
+		yell("### WARNING! Inconsistency in internal channel lists detected.");
+		yell("### For channel [%s] on server [%d]: ", 
+				channel, server);
+		yell("### The user [%s!%s] is not on the internal lists",
+				nick, uh);
+		yell("### but the person does appear to be on the channel.");
+		yell("### Please email jnelson@acronet.net giving *all* of ");
+		yell("### the information in this message.");
+		yell("### ... Adding [%s] to channel for sanity ...", nick);
+		add_to_channel(channel, nick, server, 0, 0, 0);
+	}
+
+	malloc_strcpy(&new_n->userhost, uh);
 }
 
 
@@ -1288,16 +1304,12 @@ void	move_window_channels (Window *window)
 char *	create_channel_list (int server)
 {
 	Channel	*tmp = NULL;
-	char	local_buffer[BIG_BUFFER_SIZE + 1];
+	char	*retval = NULL;
 
-	*local_buffer = 0;
 	while (traverse_all_channels(&tmp, server))
-	{
-		strmcat(local_buffer, tmp->channel, BIG_BUFFER_SIZE);
-		strmcat(local_buffer, " ", BIG_BUFFER_SIZE);
-	}
+		m_s3cat(&retval, space, tmp->channel);
 
-	return m_strdup(local_buffer);
+	return retval ? retval : m_strdup(empty_string);
 }
 
 void 	channel_server_delete (int i)
@@ -1337,51 +1349,34 @@ static void 	add_to_mode_list (const char *channel, const char *mode)
 	if (!channel || !*channel || !mode || !*mode)
 		return;
 
-	mptr = (struct modelist *) new_malloc(sizeof(struct modelist));
-	mptr->chan = (char *) 0;
-	mptr->mode = (char *) 0;
-	mptr->next = mode_list;
-	mode_list = mptr;
-	malloc_strcpy(&mptr->chan, channel);
-	malloc_strcpy(&mptr->mode, mode);
+	mptr = (Modelist *)new_malloc(sizeof(Modelist));
+	mptr->chan = m_strdup(channel);
+	mptr->mode = m_strdup(mode);
+	add_to_list((List **)&mode_list, (List *)mptr);
 }
 
 static void 	check_mode_list_join (const char *channel)
 {
-	struct modelist *mptr = mode_list;
+	Modelist *mptr;
 
-	for (; mptr; mptr = mptr->next)
+	if ((mptr = (Modelist *)remove_from_list((List **)&mode_list, channel)))
 	{
-		if (!my_stricmp(mptr->chan, channel))
-		{
-			send_to_server("MODE %s %s", mptr->chan, mptr->mode);
-			return;
-		}
+		send_to_server("MODE %s %s", mptr->chan, mptr->mode);
+		new_free(&mptr->chan);
+		new_free(&mptr->mode);
+		new_free((char **)&mptr);
 	}
-	remove_from_mode_list(channel);
 }
 
 void 	remove_from_mode_list (const char *channel)
 {
-	struct modelist *curr, *prev = (struct modelist *) 0, *next;
+	Modelist *mptr;
 
-	for (next = mode_list; next; )
+	if ((mptr = (Modelist *)remove_from_list((List **)&mode_list, channel)))
 	{
-		curr = next;
-		next = curr->next;
-		if (!my_stricmp(curr->chan, channel))
-		{
-			if (curr == mode_list)
-				mode_list = curr->next;
-			else
-				prev->next = curr->next;
-			prev = curr;
-			new_free(&curr->chan);
-			new_free(&curr->mode);
-			new_free((char **)&curr);
-		}
-		else
-			prev = curr;
+		new_free(&mptr->chan);
+		new_free(&mptr->mode);
+		new_free((char **)&mptr);
 	}
 }
 
@@ -1391,19 +1386,19 @@ void 	remove_from_mode_list (const char *channel)
  */
 int 	auto_rejoin_callback (void *d)
 {
-	char *data     = (char *) d;
-	char *channel  = next_arg(data, &data);
-	int server     = my_atol(next_arg(data, &data));
-	Window *window = get_window_by_refnum(my_atol(next_arg(data, &data)));
-	char *key      = next_arg(data, &data);
-	int old_from_server = from_server;
+	char *	data    = (char *) d;
+	char *	channel	= next_arg(data, &data);
+	int 	server	= my_atol(next_arg(data, &data));
+	Window *window 	= get_window_by_refnum(my_atol(next_arg(data, &data)));
+	char *	key    	= next_arg(data, &data);
+	int 	ofs	= from_server;
 
 	from_server = server;
 	if (key && *key)
 		send_to_server("JOIN %s %s", channel, key);
 	else
 		send_to_server("JOIN %s", channel);
-	from_server = old_from_server;
+	from_server = ofs;
 
 	if (window)
 		malloc_strcpy(&window->waiting_channel, channel);
