@@ -521,9 +521,14 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 * creation delay), we have to make sure that this numeric 
 		 * is for us and not for the channel we just joined.
 		 */
-		if ((*ArgList[0] == '#') || (*ArgList[0] == '&'))
-			if (get_server_nickname(from_server))
-				break;
+		if (is_channel(*ArgList))
+		{
+			/* Remove it so the client isn't confused */
+			remove_channel(*ArgList, from_server);
+			if (do_hook(current_numeric, "%s %s", from, *ArgList))
+				display_msg(from, ArgList);
+			break;
+		}
 
 		/* FALLTHROUGH */
 	}
@@ -547,6 +552,23 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	case 442:		/* #define ERR_NOTONCHANNEL	442 */
+	{
+		const char *	s;
+		if (!ArgList[0])
+			break;
+
+		PasteArgs(ArgList, 0);
+		s = get_server_name(from_server);
+		if (!my_strnicmp(s, from, strlen(s)))
+		{
+			if (strcmp(ArgList[0], "*"))
+			    remove_channel(ArgList[0], from_server);
+		}
+		put_it("%s %s", numeric_banner(), ArgList[0]);
+		break;
+	}
+
 	case 451:		/* #define ERR_NOTREGISTERED    451 */
 	/*
 	 * Sometimes the server doesn't catch the USER line, so
@@ -610,8 +632,6 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 * connected to a server, then doing say() is not
 		 * a good idea.  So now it just doesnt do anything.
 		 */
-		if (!is_server_connected(from_server))
-			break;
 		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 			display_msg(from, ArgList);
 		close_server(from_server, empty_string);
