@@ -108,6 +108,7 @@ char *	PasteArgs (char **arg_list, int paste_point)
 void 	BreakArgs (char *Input, char **Sender, char **OutPut)
 {
 	int	ArgCount;
+	char	*fuh;
 
 	/*
 	 * Paranoia.  Clean out any bogus ptrs still left on OutPut...
@@ -135,17 +136,19 @@ void 	BreakArgs (char *Input, char **Sender, char **OutPut)
 		/*
 		 * Look to see if the optional !user@host is present.
 		 */
-		FromUserHost = *Sender;
-		while (*FromUserHost && *FromUserHost != '!')
-			FromUserHost++;
-		if (*FromUserHost == '!')
-			*FromUserHost++ = 0;
+		fuh = *Sender;
+		while (*fuh && *fuh != '!')
+			fuh++;
+		if (*fuh == '!')
+			*fuh++ = 0;
 	}
 	/*
 	 * No sender present.
 	 */
 	else
-		*Sender = FromUserHost = empty_string;
+		*Sender = fuh = empty_string;
+
+	FromUserHost = fuh;
 
 	/*
 	 * Now we go through the argument list...
@@ -212,17 +215,16 @@ static void p_topic (char *from, char **ArgList)
 
 static void p_wallops (char *from, char **ArgList)
 {
-	char	*line;
 	int 	server_wallop = strchr(from, '.') ? 1 : 0;
 
-	if (!(line = PasteArgs(ArgList, 0)))
+	if (!ArgList[0])
 	{
 		fake();
 		return;
 	}
 
 	/* wallops from a server */
-	if (server_wallop || check_flooding(from, WALLOP_FLOOD, line))
+	if (server_wallop || check_flooding(from, WALLOP_FLOOD, ArgList[0]))
 	{
 		int	level;
 		char	*high;
@@ -243,11 +245,11 @@ static void p_wallops (char *from, char **ArgList)
 		if (do_hook(WALLOP_LIST, "%s %c %s", 
 					from, 
 					server_wallop ? 'S' : '*', 
-					line))
+					ArgList[0]))
 			put_it("%s!%s%s!%s %s", 
 					high, from, 
 					server_wallop ? empty_string : star, 
-					high, line);
+					high, ArgList[0]);
 
 		if (beep_on_level & LOG_WALLOP)
 			beep_em(1);
@@ -1140,5 +1142,6 @@ void 	parse_server (char *line)
 			rfc1459_odd(from, comm, ArgList);
 	}
 
+	FromUserHost = empty_string;
 	from_server = -1;
 }
