@@ -831,41 +831,57 @@ static	int	add_to_scratch_window_display_list (Window *window, const unsigned ch
 		*my_line_prev;
 	int 	cnt;
 
+	/* Must be a scratch window */
 	if (window->scratch_line == -1)
 		panic("This is not a scratch window.");
 
+	/* Must be within the bounds of the size of the window */
 	if (window->scratch_line > window->display_size)
 		panic("scratch_line is too big.");
 
 	my_line = window->top_of_display;
 	my_line_prev = NULL;
 
+	/* 
+	 * 'window->scratch_line' is set to the place where the next line
+	 * of output will be placed.  However, the actual contents of the
+	 * screen's display is allocated on the fly, and display_ip might
+	 * closer than 'scratch_line' entries from 'top_of_display'.  So to
+	 * forstall that possibility, this auto-grows the window display so
+	 * that it contains at least 'scratch_line' entries.
+	 */
 	for (cnt = 0; cnt < window->scratch_line; cnt++)
 	{
-		if (my_line == window->display_ip)
+	    if (my_line == window->display_ip)
+	    {
+		if (my_line_prev)
 		{
-			if (my_line_prev)
-			{
-				my_line_prev->next = new_display_line(my_line_prev);
-				my_line = my_line_prev->next;
-				my_line->next = window->display_ip;
-				window->display_ip->prev = my_line;
-				window->display_buffer_size++;
-			}
-			else
-			{
-				window->top_of_display = new_display_line(my_line_prev);
-				my_line = window->top_of_display;
-				my_line->next = window->display_ip;
-				window->display_ip->prev = my_line;
-				window->display_buffer_size++;
-			}
+			my_line_prev->next = new_display_line(my_line_prev);
+			my_line = my_line_prev->next;
+			my_line->next = window->display_ip;
+			window->display_ip->prev = my_line;
+			window->display_buffer_size++;
 		}
+		else
+		{
+			window->top_of_display = new_display_line(my_line_prev);
+			my_line = window->top_of_display;
+			my_line->next = window->display_ip;
+			window->display_ip->prev = my_line;
+			window->display_buffer_size++;
+		}
+	    }
 
-		my_line_prev = my_line;
-		my_line = my_line->next;
+	    my_line_prev = my_line;
+	    my_line = my_line->next;
 	}
-	/* XXXX DO this right!!!! XXXX */
+
+	/*
+	 * Except that we only actually create the window display to be
+	 * one entry less than it should be, so at this point, if my_line
+	 * is the same as display_ip, then we have to create one more
+	 * entry to hold the data.  XXX This is not handled very elegantly.
+	 */
 	if (my_line == window->display_ip)
 	{
 		my_line = new_display_line(window->display_ip->prev);
@@ -876,25 +892,55 @@ static	int	add_to_scratch_window_display_list (Window *window, const unsigned ch
 		window->display_buffer_size++;
 	}
 
+	/*
+	 * Alright then!  Copy the line of output into the buffer, and
+	 * set the cursor to just after this line.
+	 */
 	malloc_strcpy(&my_line->line, str);
 	window->cursor = window->scratch_line;
 
 	/*
-	 * Make sure the cursor ends up in the right spot
+	 * Increment the scratch line to the next location.
+	 */
+	window->scratch_line++;
+	if (window->scratch_line >= window->display_size)
+		window->scratch_line = 0;	/* Just go back to top */
+
+	/*
+	 * For /window scroll off, clear the next line as long as it is
+	 * not the display_ip entry.
+	 */
+	if (window->noscroll)
+	{
+		if (window->scratch_line == 0)
+			my_line = window->top_of_display;
+		else if (my_line->next != window->display_ip)
+			my_line = my_line->next;
+		else
+			my_line = NULL;
+
+		if (my_line)
+			malloc_strcpy(&my_line->line, empty_string);
+	}
+
+	/*
+	 * If this is a visible window, output the new line.
+	 * If this is a non-scrolling window, clear the next line.
 	 */
 	if (window->visible)
 	{
-		window->cursor = window->scratch_line;
 		window->screen->cursor_window = window;
 		term_move_cursor(0, window->top + window->cursor);
 		term_clear_to_eol();
 		output_line(str);
 		display_standout(OFF);
+		if (window->noscroll)
+		{
+		    term_move_cursor(0, window->top + window->scratch_line);
+		    term_clear_to_eol();
+		}
 	}
 
-	window->scratch_line++;
-	if (window->scratch_line >= window->display_size)
-		window->scratch_line = 0;	/* Just go back to top */
 	window->cursor = window->scratch_line;
 	return 0;		/* Express a failure */
 }
@@ -1689,7 +1735,16 @@ Window	*create_additional_screen (void)
 
 	oldscreen = current_window->screen;
 	new_s = create_new_screen();
-
+	/*
+	 * At this point, doing a say() or yell() or anything else that would
+	 * output to the screen will cause a refresh of the status bar and
+	 * input line.  new_s->current_window is NULL after the above line,
+	 * so any attempt to reference $C or $T will be to NULL pointers,
+	 * which will cause a crash.  For various reasons, we can't fire up
+	 * a new window this early, so its just easier to make sure we don't
+	 * output anything until kill_screen() or new_window() is called first.
+	 * You have been warned!
+	 */
 	switch ((child = fork()))
 	{
 		case -1:
@@ -1805,8 +1860,8 @@ Window	*create_additional_screen (void)
 				accept(s, (struct sockaddr *) &NewSock, &NsZ);
 		if (new_s->fdin < 0)
 		{
-			yell("Couldn't accept the new connection!");
 			kill_screen(new_s);
+			yell("Couldn't accept the new connection!");
 			return (Window *) 0;
 		}
 		new_open(new_s->fdin);
@@ -1820,13 +1875,12 @@ Window	*create_additional_screen (void)
 		FD_SET(new_s->fdin, &fd_read);
 		select(NFDBITS, &fd_read, NULL, NULL, &timeout);
 
-		yell("Checking for a wserv on fd [%d]", new_s->fdin);
 		if ((s = dgets(buffer, new_s->fdin, 0)) < 1)
 		{
+			kill_screen(new_s);
 			yell("Read [%s] from wserv, but dgets() returned [%d]",
 				buffer, s);
 			new_s->fdin = new_close(new_s->fdin);
-			kill_screen(new_s);
 			kill(child, SIGKILL);
 			return NULL;
 		}
@@ -1917,7 +1971,8 @@ void 	do_screens (fd_set *rd)
 					irc_exit(1, "EPIC - EOF from stdin");
 				}
 
-				*(buffer + strlen(buffer) - 1) = '\0';
+				if (strlen(buffer))
+					buffer[strlen(buffer) - 1] = 0;
 				if (get_int_var(INPUT_ALIASES_VAR))
 					parse_line(NULL, buffer, empty_string, 1, 0);
 				else
@@ -2483,8 +2538,11 @@ unsigned char *strip_ansi (const unsigned char *str)
 			 */
 			case ('G'):
 			{
-				while ((chr = next_char()) != ':')
+				while ((chr = next_char()) != 0 &&
+					chr != ':')
 					;
+				if (chr == 0)
+					put_back();
 				break;
 			}
 
@@ -2494,8 +2552,11 @@ unsigned char *strip_ansi (const unsigned char *str)
 			 */
 			case ('P') :
 			{
-				while ((chr = next_char()) != 033)
+				while ((chr = next_char()) != 0 &&
+					chr != 033)
 					;
+				if (chr == 0)
+					put_back();
 				break;
 			}
 
@@ -2515,6 +2576,8 @@ unsigned char *strip_ansi (const unsigned char *str)
 			{
 				while ((chr = next_char()) != 0 && chr != 7)
 					;
+				if (chr == 0)
+					put_back();
 				break;
 			}
 
@@ -2966,17 +3029,20 @@ static	int	last_fore = -2,
 	if (back == -1)
 		back = last_back;
 
+	if (fore > -1 && fore_conv[fore] < 8)
+		strcat(retbuf, TI_sgrstrs[TERM_SGR_BOLD_OFF - 1]);
+	if (back > -1 && back < 58)
+		if (fore_conv[back] < 8 &&
+		    get_int_var(TERM_DOES_BRIGHT_BLINK_VAR))
+			strcat(retbuf, TI_sgrstrs[TERM_SGR_BLINK_OFF - 1]);
+
 	if (back == 58)
 		strcat(retbuf, TI_sgrstrs[TERM_SGR_BLINK_ON - 1]);
 	if (fore > -1)
-	{
-		if (fore_conv[fore] < 8 /* && (x_debug & DEBUG_BOLD_HELPER) */)
-			strcat(retbuf, TI_sgrstrs[TERM_SGR_BOLD_OFF-1]);
 		strcat(retbuf, TI_forecolors[fore_conv[fore]]);
-	}
-	if (back > -1 && back != 58)
+	if (back > -1 && back < 58)
 	{
-		if (back < 17 && get_int_var(TERM_DOES_BRIGHT_BLINK_VAR))
+		if (get_int_var(TERM_DOES_BRIGHT_BLINK_VAR))
 			strcat(retbuf, TI_backcolors[fore_conv[back]]);
 		else
 			strcat(retbuf, TI_backcolors[back_conv[back]]);
