@@ -70,6 +70,26 @@ static	Char	*status_user16 		(Window *);
 static	Char	*status_user17 		(Window *);
 static	Char	*status_user18 		(Window *);
 static	Char	*status_user19 		(Window *);
+static	Char	*status_user20 		(Window *);
+static	Char	*status_user21 		(Window *);
+static	Char	*status_user22 		(Window *);
+static	Char	*status_user23 		(Window *);
+static	Char	*status_user24 		(Window *);
+static	Char	*status_user25 		(Window *);
+static	Char	*status_user26 		(Window *);
+static	Char	*status_user27 		(Window *);
+static	Char	*status_user28 		(Window *);
+static	Char	*status_user29 		(Window *);
+static	Char	*status_user30 		(Window *);
+static	Char	*status_user31 		(Window *);
+static	Char	*status_user32 		(Window *);
+static	Char	*status_user33 		(Window *);
+static	Char	*status_user34 		(Window *);
+static	Char	*status_user35 		(Window *);
+static	Char	*status_user36 		(Window *);
+static	Char	*status_user37 		(Window *);
+static	Char	*status_user38 		(Window *);
+static	Char	*status_user39 		(Window *);
 static	Char	*status_dcc 		(Window *);
 static	Char	*status_hold 		(Window *);
 static	Char	*status_version 	(Window *);
@@ -168,7 +188,27 @@ struct status_formats status_expandos[] = {
 { 1, '7', status_user17,	 NULL, 			-1 },
 { 1, '8', status_user18,	 NULL, 			-1 },
 { 1, '9', status_user19,	 NULL, 			-1 },
-{ 1, 'T', status_test,		 NULL,			-1 }
+{ 1, 'T', status_test,		 NULL,			-1 },
+{ 2, '0', status_user20,	 NULL, 			-1 },
+{ 2, '1', status_user21,	 NULL, 			-1 },
+{ 2, '2', status_user22,	 NULL, 			-1 },
+{ 2, '3', status_user23,	 NULL, 			-1 },
+{ 2, '4', status_user24,	 NULL, 			-1 },
+{ 2, '5', status_user25,	 NULL, 			-1 },
+{ 2, '6', status_user26,	 NULL, 			-1 },
+{ 2, '7', status_user27,	 NULL, 			-1 },
+{ 2, '8', status_user28,	 NULL, 			-1 },
+{ 2, '9', status_user29,	 NULL, 			-1 },
+{ 3, '0', status_user30,	 NULL, 			-1 },
+{ 3, '1', status_user31,	 NULL, 			-1 },
+{ 3, '2', status_user32,	 NULL, 			-1 },
+{ 3, '3', status_user33,	 NULL, 			-1 },
+{ 3, '4', status_user34,	 NULL, 			-1 },
+{ 3, '5', status_user35,	 NULL, 			-1 },
+{ 3, '6', status_user36,	 NULL, 			-1 },
+{ 3, '7', status_user37,	 NULL, 			-1 },
+{ 3, '8', status_user38,	 NULL, 			-1 },
+{ 3, '9', status_user39,	 NULL, 			-1 }
 };
 #define NUMBER_OF_EXPANDOS (sizeof(status_expandos) / sizeof(struct status_formats))
 
@@ -807,7 +847,8 @@ static	char	my_buffer[64];
 	if (!strncmp(name, "irc", 3))
 	{
 		name = rest + 1;
-		rest = strchr(name + 1, '.');
+		if (!(rest = strchr(name + 1, '.')))
+			rest = name + strlen(name);
 	}
 
 	/*
@@ -1153,6 +1194,26 @@ STATUS_VAR(5) STATUS_VAR(6) STATUS_VAR(7) STATUS_VAR(8) STATUS_VAR(9)
 STATUS_VAR(10) STATUS_VAR(11) STATUS_VAR(12) STATUS_VAR(13) STATUS_VAR(14)
 STATUS_VAR(15) STATUS_VAR(16) STATUS_VAR(17) STATUS_VAR(18) STATUS_VAR(19)
 
+/*
+ * This is a generic status_userX variable.  All of these go to all the
+ * windows, by popular request.
+ */
+#define STATUS_VVAR(x) \
+static	Char	*status_user ## x ## (Window *window)			\
+{									\
+	char	*text;							\
+									\
+	if ((text = get_string_var(STATUS_USER ## x ## _VAR)))		\
+		return text;						\
+	else								\
+		return empty_string;					\
+}
+
+STATUS_VVAR(20) STATUS_VVAR(21) STATUS_VVAR(22) STATUS_VVAR(23) STATUS_VVAR(24)
+STATUS_VVAR(25) STATUS_VVAR(26) STATUS_VVAR(27) STATUS_VVAR(28) STATUS_VVAR(29)
+STATUS_VVAR(30) STATUS_VVAR(31) STATUS_VVAR(32) STATUS_VVAR(33) STATUS_VVAR(34)
+STATUS_VVAR(35) STATUS_VVAR(36) STATUS_VVAR(37) STATUS_VVAR(38) STATUS_VVAR(39)
+
 static	Char	*status_hold (Window *window)
 {
 	char *text;
