@@ -72,6 +72,7 @@ static 	void	set_dcc_timeout (int value);
 static	void	set_mangle_inbound (char *value);
 static	void	set_mangle_outbound (char *value);
 static	void	set_mangle_logfiles (char *value);
+static	void	set_scroll (int value);
 
 
 /*
@@ -84,9 +85,9 @@ static	IrcVariable irc_variable[] =
 	{ "ALWAYS_SPLIT_BIGGEST",	BOOL_TYPE_VAR,	DEFAULT_ALWAYS_SPLIT_BIGGEST, NULL, NULL, 0, 0 },
 	{ "AUTO_NEW_NICK",		BOOL_TYPE_VAR,	DEFAULT_AUTO_NEW_NICK, NULL, NULL, 0, 0 },
         { "AUTO_RECONNECT",             BOOL_TYPE_VAR,  DEFAULT_AUTO_RECONNECT, NULL, NULL, 0, 0 },
-	{ "AUTO_RECONNECT_DELAY_VAR",	INT_TYPE_VAR,	DEFAULT_AUTO_RECONNECT_DELAY, NULL, NULL, 0, 0 },
+	{ "AUTO_RECONNECT_DELAY",	INT_TYPE_VAR,	DEFAULT_AUTO_RECONNECT_DELAY, NULL, NULL, 0, 0 },
         { "AUTO_REJOIN",                BOOL_TYPE_VAR,  DEFAULT_AUTO_REJOIN, NULL, NULL, 0, 0 },
-	{ "AUTO_REJOIN_DELAY_VAR",	INT_TYPE_VAR,	DEFAULT_AUTO_REJOIN_DELAY, NULL, NULL, 0, 0 },
+	{ "AUTO_REJOIN_DELAY",		INT_TYPE_VAR,	DEFAULT_AUTO_REJOIN_DELAY, NULL, NULL, 0, 0 },
 	{ "AUTO_UNMARK_AWAY",		BOOL_TYPE_VAR,	DEFAULT_AUTO_UNMARK_AWAY, NULL, NULL, 0, 0 },
 	{ "AUTO_WHOWAS",		BOOL_TYPE_VAR,	DEFAULT_AUTO_WHOWAS, NULL, NULL, 0, 0 },
 	{ "BAD_STYLE",			BOOL_TYPE_VAR,	DEFAULT_BAD_STYLE, NULL, NULL, 0, 0 },
@@ -104,7 +105,7 @@ static	IrcVariable irc_variable[] =
 	{ "CLOCK_24HOUR",		BOOL_TYPE_VAR,	DEFAULT_CLOCK_24HOUR, NULL, reset_clock, 0, 0 },
 	{ "CLOCK_FORMAT",		STR_TYPE_VAR,	0, NULL, set_clock_format, 0, 0 },
 	{ "CMDCHARS",			STR_TYPE_VAR,	0, NULL, NULL, 0, 0 },
-	{ "COLOR",			BOOL_TYPE_VAR,	DEFAULT_COLOR, NULL, 0, 0 },
+	{ "COLOR",			BOOL_TYPE_VAR,	DEFAULT_COLOR, NULL, NULL, 0, 0 },
 	{ "COMMAND_MODE",		BOOL_TYPE_VAR,	DEFAULT_COMMAND_MODE, NULL, NULL, 0, 0 },
 	{ "COMMENT_HACK",		BOOL_TYPE_VAR,	DEFAULT_COMMENT_HACK, NULL, NULL, 0, 0 },
 	{ "CONNECT_TIMEOUT",		INT_TYPE_VAR,	DEFAULT_CONNECT_TIMEOUT, NULL, NULL, 0, 0 },
@@ -123,7 +124,7 @@ static	IrcVariable irc_variable[] =
 	{ "DISPLAY_PC_CHARACTERS",	INT_TYPE_VAR,	DEFAULT_DISPLAY_PC_CHARACTERS, NULL, set_display_pc_characters, 0, 0 },
 	{ "DO_NOTIFY_IMMEDIATELY",	BOOL_TYPE_VAR,	DEFAULT_DO_NOTIFY_IMMEDIATELY, NULL, NULL, 0, 0 },
 	{ "EIGHT_BIT_CHARACTERS",	BOOL_TYPE_VAR,	DEFAULT_EIGHT_BIT_CHARACTERS, NULL, eight_bit_characters, 0, 0 },
-	{ "FLOATING_POINT_MATH",	BOOL_TYPE_VAR,	DEFAULT_FLOATING_POINT_MATH, NULL, 0, 0 },
+	{ "FLOATING_POINT_MATH",	BOOL_TYPE_VAR,	DEFAULT_FLOATING_POINT_MATH, NULL, NULL, 0, 0 },
 	{ "FLOOD_AFTER",		INT_TYPE_VAR,	DEFAULT_FLOOD_AFTER, NULL, NULL, 0, 0 },
 	{ "FLOOD_RATE",			INT_TYPE_VAR,	DEFAULT_FLOOD_RATE, NULL, NULL, 0, 0 },
 	{ "FLOOD_USERS",		INT_TYPE_VAR,	DEFAULT_FLOOD_USERS, NULL, NULL, 0, 0 },
@@ -163,7 +164,7 @@ static	IrcVariable irc_variable[] =
 	{ "NOTIFY_LEVEL",		STR_TYPE_VAR,	0, NULL, set_notify_level, 0, 0 },
 	{ "NOTIFY_ON_TERMINATION",	BOOL_TYPE_VAR,	DEFAULT_NOTIFY_ON_TERMINATION, NULL, NULL, 0, 0 },
 	{ "NOTIFY_USERHOST_AUTOMATIC",	BOOL_TYPE_VAR,	DEFAULT_NOTIFY_USERHOST_AUTOMATIC, NULL, NULL, 0, 0 },
-	{ "NO_CONTROL_LOG_VAR",		BOOL_TYPE_VAR,	DEFAULT_NO_CONTROL_LOG, NULL, NULL, 0, 0 },
+	{ "NO_CONTROL_LOG",		BOOL_TYPE_VAR,	DEFAULT_NO_CONTROL_LOG, NULL, NULL, 0, 0 },
 	{ "NO_CTCP_FLOOD",		BOOL_TYPE_VAR,	DEFAULT_NO_CTCP_FLOOD, NULL, NULL, 0, 0 },
 	{ "NO_FAIL_DISCONNECT",		BOOL_TYPE_VAR,	DEFAULT_NO_FAIL_DISCONNECT, NULL, NULL, 0, 0 },
 	{ "NUM_OF_WHOWAS",		INT_TYPE_VAR,	DEFAULT_NUM_OF_WHOWAS, NULL, NULL, 0, 0 },
@@ -174,6 +175,7 @@ static	IrcVariable irc_variable[] =
 	{ "REALNAME",			STR_TYPE_VAR,	0, NULL, set_realname, 0, 0 },
 	{ "REVERSE_STATUS_LINE",	BOOL_TYPE_VAR,	DEFAULT_REVERSE_STATUS_LINE, NULL, update_all_status, 0, 0 },
 	{ "SCREEN_OPTIONS",             STR_TYPE_VAR,   0, NULL, NULL, 0, 0 },
+	{ "SCROLL",			BOOL_TYPE_VAR,	1, NULL, set_scroll, 0, 0 },
 	{ "SCROLLBACK",			INT_TYPE_VAR,	DEFAULT_SCROLLBACK, NULL, set_scrollback_size, 0, 0 },
 	{ "SCROLLBACK_RATIO",		INT_TYPE_VAR,	DEFAULT_SCROLLBACK_RATIO, NULL, NULL, 0, 0 },
 	{ "SCROLL_LINES",		INT_TYPE_VAR,	DEFAULT_SCROLL_LINES, NULL, set_scroll_lines, 0, 0 },
@@ -223,7 +225,27 @@ static	IrcVariable irc_variable[] =
 	{ "STATUS_USER18",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_USER19",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },	
 	{ "STATUS_USER2",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER20",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER21",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER22",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER23",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER24",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER25",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER26",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER27",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER28",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER29",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },	
 	{ "STATUS_USER3",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER30",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER31",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER32",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER33",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER34",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER35",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER36",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER37",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER38",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
+	{ "STATUS_USER39",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },	
 	{ "STATUS_USER4",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_USER5",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
 	{ "STATUS_USER6",		STR_TYPE_VAR,	0, NULL, build_status, 0, 0 },
@@ -319,6 +341,26 @@ void 	init_variables (void)
 	set_string_var(STATUS_USER17_VAR, DEFAULT_STATUS_USER17);
 	set_string_var(STATUS_USER18_VAR, DEFAULT_STATUS_USER18);
 	set_string_var(STATUS_USER19_VAR, DEFAULT_STATUS_USER19);
+	set_string_var(STATUS_USER20_VAR, DEFAULT_STATUS_USER20);
+	set_string_var(STATUS_USER21_VAR, DEFAULT_STATUS_USER21);
+	set_string_var(STATUS_USER22_VAR, DEFAULT_STATUS_USER22);
+	set_string_var(STATUS_USER23_VAR, DEFAULT_STATUS_USER23);
+	set_string_var(STATUS_USER24_VAR, DEFAULT_STATUS_USER24);
+	set_string_var(STATUS_USER25_VAR, DEFAULT_STATUS_USER25);
+	set_string_var(STATUS_USER26_VAR, DEFAULT_STATUS_USER26);
+	set_string_var(STATUS_USER27_VAR, DEFAULT_STATUS_USER27);
+	set_string_var(STATUS_USER28_VAR, DEFAULT_STATUS_USER28);
+	set_string_var(STATUS_USER29_VAR, DEFAULT_STATUS_USER29);
+	set_string_var(STATUS_USER30_VAR, DEFAULT_STATUS_USER30);
+	set_string_var(STATUS_USER31_VAR, DEFAULT_STATUS_USER31);
+	set_string_var(STATUS_USER32_VAR, DEFAULT_STATUS_USER32);
+	set_string_var(STATUS_USER33_VAR, DEFAULT_STATUS_USER33);
+	set_string_var(STATUS_USER34_VAR, DEFAULT_STATUS_USER34);
+	set_string_var(STATUS_USER35_VAR, DEFAULT_STATUS_USER35);
+	set_string_var(STATUS_USER36_VAR, DEFAULT_STATUS_USER36);
+	set_string_var(STATUS_USER37_VAR, DEFAULT_STATUS_USER37);
+	set_string_var(STATUS_USER38_VAR, DEFAULT_STATUS_USER38);
+	set_string_var(STATUS_USER39_VAR, DEFAULT_STATUS_USER39);
 	set_string_var(STATUS_VOICE_VAR, DEFAULT_STATUS_VOICE);
 	set_string_var(STATUS_WINDOW_VAR, DEFAULT_STATUS_WINDOW);
 	set_string_var(USERINFO_VAR, DEFAULT_USERINFO);
@@ -874,13 +916,13 @@ int	parse_mangle (char *value, int nvalue, char **rv)
 				nvalue &= ~(STRIP_ND_SPACE);
 			else if (!my_strnicmp(str2, "NONE", 2))
 				nvalue = 0;
-			else if (!my_strnicmp(str2, "REVERSE", 1))
+			else if (!my_strnicmp(str2, "REVERSE", 2))
 				nvalue |= STRIP_REVERSE;
-			else if (!my_strnicmp(str2, "-REVERSE", 2))
+			else if (!my_strnicmp(str2, "-REVERSE", 3))
 				nvalue &= ~(STRIP_REVERSE);
-			else if (!my_strnicmp(str2, "ROM_CHAR", 1))
+			else if (!my_strnicmp(str2, "ROM_CHAR", 2))
 				nvalue |= STRIP_ROM_CHAR;
-			else if (!my_strnicmp(str2, "-ROM_CHAR", 2))
+			else if (!my_strnicmp(str2, "-ROM_CHAR", 3))
 				nvalue &= ~(STRIP_ROM_CHAR);
 			else if (!my_strnicmp(str2, "UNDERLINE", 1))
 				nvalue |= STRIP_UNDERLINE;
@@ -942,6 +984,20 @@ static	void	set_mangle_logfiles (char *value)
 	new_free(&nv);
 }
 
+static	void	set_scroll (int value)
+{
+	char 	*my_zero = "OFF";
+	char	*my_one = "ON";
+	int	owd = window_display;
+
+	window_display = 0;
+	if (value)
+		window_scroll(current_window, &my_one);
+	else
+		window_scroll(current_window, &my_zero);
+	window_display = owd;
+}
+
 /*******/
 typedef struct	varstacklist
 {
