@@ -842,7 +842,7 @@ void userhostbase(char *args, void (*line) (UserhostItem *, char *, char *), int
 			char *nick = next_arg(ptr, &ptr);
 			const char *ouh = fetch_userhost(from_server, nick);
 			char *uh = alloca(strlen(ouh) + 1);
-			UserhostItem item = {0};
+			UserhostItem item = {NULL, 0, 0, 0, NULL, NULL};
 
 			item.nick = nick;
 			item.oper = 0;
@@ -880,7 +880,7 @@ void	userhost_returned (char *from, char **ArgList)
 
 	if (!top)
 	{
-		yell("### Dont do /quote USERHOST");
+		yell("### Please don't /quote a server command that returns the 302 numeric.");
 		return;
 	}
 
@@ -941,6 +941,7 @@ void	userhost_returned (char *from, char **ArgList)
 				yell("Can't parse broken USERHOST reply [%s]", 
 						*ArgList);
 				userhost_queue_pop();
+				return;
 			}
 			*item.host++ = 0;
 
